@@ -113,7 +113,7 @@ struct config_file_t
 
 /* Functions used throughout the parser */
 AACS_PRIVATE int keydbcfg_parse_config(config_file *cfgfile, const char *path);
-AACS_PRIVATE config_file *keydbcfg_new_config_file();
+AACS_PRIVATE config_file *keydbcfg_new_config_file(void);
 AACS_PRIVATE int keydbcfg_config_file_close(config_file *cfgfile);
 
 /* */
