@@ -73,99 +73,9 @@
 
 /* Copy the first part of user declarations.  */
 
-/* Line 268 of yacc.c  */
-#line 1 "keydbcfg-parser.y"
-
-/*
- * This file is part of libaacs
- * Copyright (C) 2010  gates
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see
- * <http://www.gnu.org/licenses/>.
- */
-
-#include "file/keydbcfg.h"
-#include "util/macro.h"
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-/* Disable some warnings triggered by generated parser */
-#if defined __GNUC__
-#pragma GCC diagnostic ignored "-Wredundant-decls"
-#pragma GCC diagnostic ignored "-Wimplicit-function-declaration"
-#pragma GCC diagnostic ignored "-Wunused-parameter"
-#endif
-
-/* Fix some warnings trigger by -Wundef which can't be ignored */
-#define YYENABLE_NLS 0
-#define YYLTYPE_IS_TRIVIAL 0
-
-#define DIGIT_KEY_PAIR_LIST_FREE(X) do   \
-{                                        \
-  while (X)                              \
-  {                                      \
-    digit_key_pair_list *next = X->next; \
-    X_FREE(X->key_pair.key);             \
-    X_FREE(X);                           \
-    X = next;                            \
-  }                                      \
-} while (0);
-
-/* enum used in certain functions to add proper title entry */
-enum
-{
-  ENTRY_TYPE_DISCID,
-  ENTRY_TYPE_TITLE,
-  ENTRY_TYPE_DATE,
-  ENTRY_TYPE_MEK,
-  ENTRY_TYPE_VID,
-  ENTRY_TYPE_BN,
-  ENTRY_TYPE_VUK,
-  ENTRY_TYPE_PAK,
-  ENTRY_TYPE_TK,
-  ENTRY_TYPE_UK
-};
-
-static dk_list *new_dk_list();
-static dk_list *add_dk_list_entry(dk_list *list, const char *key,
-                                  const char *node);
-static pk_list *new_pk_list();
-static pk_list *add_pk_list_entry(pk_list *list, const char *key);
-static cert_list *new_cert_list();
-static cert_list *add_cert_list(cert_list *list, const char *host_priv_key,
-                         const char *host_cert, const char *host_nonce,
-                         const char *host_key_point);
-static title_entry_list *new_title_entry_list();
-static int add_entry(title_entry_list *list, int type, const char *entry);
-static digit_key_pair_list *new_digit_key_pair_list();
-static digit_key_pair_list *add_digit_key_pair_entry(digit_key_pair_list *list,
-                              int type, unsigned int digit, const char *key);
-static int add_date_entry(title_entry_list *list, unsigned int year,
-                          unsigned int month, unsigned int day);
-void yyerror (void *scanner, dk_list *dklist, pk_list *pklist, cert_list *clist,
-              title_entry_list *celist, digit_key_pair_list *dkplist,
-              const char *msg);
-extern int libaacs_yyget_lineno  (void *scanner);
-
-/* uncomment the line below for debugging */
-// int yydebug = 1;
-
 
 /* Line 268 of yacc.c  */
-#line 169 "keydbcfg-parser.c"
+#line 79 "keydbcfg-parser.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -185,6 +95,17 @@ extern int libaacs_yyget_lineno  (void *scanner);
 # define YYTOKEN_TABLE 0
 #endif
 
+/* "%code requires" blocks.  */
+
+/* Line 288 of yacc.c  */
+#line 1 "keydbcfg-parser.y"
+
+#include "file/keydbcfg.h"
+
+
+
+/* Line 288 of yacc.c  */
+#line 109 "keydbcfg-parser.c"
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -254,7 +175,7 @@ typedef union YYSTYPE
 {
 
 /* Line 293 of yacc.c  */
-#line 106 "keydbcfg-parser.y"
+#line 108 "keydbcfg-parser.y"
 
   char *string;
   unsigned int digit;
@@ -262,7 +183,7 @@ typedef union YYSTYPE
 
 
 /* Line 293 of yacc.c  */
-#line 266 "keydbcfg-parser.c"
+#line 187 "keydbcfg-parser.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -274,7 +195,101 @@ typedef union YYSTYPE
 
 
 /* Line 343 of yacc.c  */
-#line 278 "keydbcfg-parser.c"
+#line 199 "keydbcfg-parser.c"
+/* Unqualified %code blocks.  */
+
+/* Line 344 of yacc.c  */
+#line 5 "keydbcfg-parser.y"
+
+/*
+ * This file is part of libaacs
+ * Copyright (C) 2010  gates
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, see
+ * <http://www.gnu.org/licenses/>.
+ */
+
+#include "util/macro.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+/* Disable some warnings triggered by generated parser */
+#if defined __GNUC__
+#pragma GCC diagnostic ignored "-Wimplicit-function-declaration"
+#pragma GCC diagnostic ignored "-Wunused-parameter"
+#pragma GCC visibility push(hidden)
+#endif
+
+/* Fix some warnings trigger by -Wundef which can't be ignored */
+#define YYENABLE_NLS 0
+#define YYLTYPE_IS_TRIVIAL 0
+
+#define DIGIT_KEY_PAIR_LIST_FREE(X) do   \
+{                                        \
+  while (X)                              \
+  {                                      \
+    digit_key_pair_list *pnext = X->next;\
+    X_FREE(X->key_pair.key);             \
+    X_FREE(X);                           \
+    X = pnext;                           \
+  }                                      \
+} while (0);
+
+/* enum used in certain functions to add proper title entry */
+enum
+{
+  ENTRY_TYPE_DISCID,
+  ENTRY_TYPE_TITLE,
+  ENTRY_TYPE_DATE,
+  ENTRY_TYPE_MEK,
+  ENTRY_TYPE_VID,
+  ENTRY_TYPE_BN,
+  ENTRY_TYPE_VUK,
+  ENTRY_TYPE_PAK,
+  ENTRY_TYPE_TK,
+  ENTRY_TYPE_UK
+};
+
+static dk_list *new_dk_list(void);
+static dk_list *add_dk_list_entry(dk_list *list, char *key, char *node);
+static pk_list *new_pk_list(void);
+static pk_list *add_pk_list_entry(pk_list *list, char *key);
+static cert_list *new_cert_list(void);
+static cert_list *add_cert_list(cert_list *list, char *host_priv_key,
+                         char *host_cert, char *host_nonce,
+                         char *host_key_point);
+static title_entry_list *new_title_entry_list(void);
+static int add_entry(title_entry_list *list, int type, char *entry);
+static digit_key_pair_list *new_digit_key_pair_list(void);
+static digit_key_pair_list *add_digit_key_pair_entry(digit_key_pair_list *list,
+                              int type, unsigned int digit, char *key);
+static int add_date_entry(title_entry_list *list, unsigned int year,
+                          unsigned int month, unsigned int day);
+void yyerror (void *scanner, dk_list *dklist, pk_list *pklist, cert_list *clist,
+              title_entry_list *celist, digit_key_pair_list *dkplist,
+              const char *msg);
+extern int libaacs_yyget_lineno  (void *scanner);
+
+/* uncomment the line below for debugging */
+// int yydebug = 1;
+
+
+
+/* Line 344 of yacc.c  */
+#line 293 "keydbcfg-parser.c"
 
 #ifdef short
 # undef short
@@ -588,13 +603,13 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   146,   146,   147,   148,   149,   153,   154,   158,   159,
-     160,   161,   162,   171,   175,   182,   187,   192,   196,   203,
-     207,   214,   219,   224,   229,   234,   239,   247,   248,   252,
-     260,   264,   268,   269,   273,   274,   275,   276,   277,   278,
-     279,   280,   284,   291,   298,   305,   312,   313,   317,   329,
-     336,   343,   344,   348,   360,   367,   368,   372,   384,   391,
-     392,   396,   408,   416
+       0,   148,   148,   149,   150,   151,   155,   156,   160,   161,
+     162,   163,   164,   173,   177,   184,   189,   194,   198,   205,
+     209,   216,   221,   226,   231,   236,   241,   249,   250,   254,
+     262,   266,   270,   271,   275,   276,   277,   278,   279,   280,
+     281,   282,   286,   293,   300,   307,   314,   315,   319,   331,
+     338,   345,   346,   350,   362,   369,   370,   374,   386,   393,
+     394,   398,   410,   418
 };
 #endif
 
@@ -1651,7 +1666,7 @@ yyreduce:
         case 12:
 
 /* Line 1806 of yacc.c  */
-#line 163 "keydbcfg-parser.y"
+#line 165 "keydbcfg-parser.y"
     {
       fprintf(stderr, "bad entry at or around line %d\n",
               libaacs_yyget_lineno(scanner) - 1);
@@ -1662,7 +1677,7 @@ yyreduce:
   case 13:
 
 /* Line 1806 of yacc.c  */
-#line 172 "keydbcfg-parser.y"
+#line 174 "keydbcfg-parser.y"
     {
       dklist = add_dk_list_entry(dklist, (yyvsp[(3) - (6)].string), (yyvsp[(5) - (6)].string));
     }
@@ -1671,7 +1686,7 @@ yyreduce:
   case 14:
 
 /* Line 1806 of yacc.c  */
-#line 176 "keydbcfg-parser.y"
+#line 178 "keydbcfg-parser.y"
     {
       dklist = add_dk_list_entry(dklist, (yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].string));
     }
@@ -1680,21 +1695,21 @@ yyreduce:
   case 15:
 
 /* Line 1806 of yacc.c  */
-#line 183 "keydbcfg-parser.y"
+#line 185 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 16:
 
 /* Line 1806 of yacc.c  */
-#line 188 "keydbcfg-parser.y"
+#line 190 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 17:
 
 /* Line 1806 of yacc.c  */
-#line 193 "keydbcfg-parser.y"
+#line 195 "keydbcfg-parser.y"
     {
       pklist = add_pk_list_entry(pklist, (yyvsp[(3) - (4)].string));
     }
@@ -1703,7 +1718,7 @@ yyreduce:
   case 18:
 
 /* Line 1806 of yacc.c  */
-#line 197 "keydbcfg-parser.y"
+#line 199 "keydbcfg-parser.y"
     {
       pklist = add_pk_list_entry(pklist, (yyvsp[(2) - (3)].string));
     }
@@ -1712,7 +1727,7 @@ yyreduce:
   case 19:
 
 /* Line 1806 of yacc.c  */
-#line 204 "keydbcfg-parser.y"
+#line 206 "keydbcfg-parser.y"
     {
       clist = add_cert_list(clist, (yyvsp[(3) - (10)].string), (yyvsp[(5) - (10)].string), (yyvsp[(7) - (10)].string), (yyvsp[(9) - (10)].string));
     }
@@ -1721,7 +1736,7 @@ yyreduce:
   case 20:
 
 /* Line 1806 of yacc.c  */
-#line 208 "keydbcfg-parser.y"
+#line 210 "keydbcfg-parser.y"
     {
       clist = add_cert_list(clist, (yyvsp[(2) - (9)].string), (yyvsp[(4) - (9)].string), (yyvsp[(6) - (9)].string), (yyvsp[(8) - (9)].string));
     }
@@ -1730,35 +1745,35 @@ yyreduce:
   case 21:
 
 /* Line 1806 of yacc.c  */
-#line 215 "keydbcfg-parser.y"
+#line 217 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 22:
 
 /* Line 1806 of yacc.c  */
-#line 220 "keydbcfg-parser.y"
+#line 222 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 23:
 
 /* Line 1806 of yacc.c  */
-#line 225 "keydbcfg-parser.y"
+#line 227 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 24:
 
 /* Line 1806 of yacc.c  */
-#line 230 "keydbcfg-parser.y"
+#line 232 "keydbcfg-parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 25:
 
 /* Line 1806 of yacc.c  */
-#line 235 "keydbcfg-parser.y"
+#line 237 "keydbcfg-parser.y"
     {
       celist->next = new_title_entry_list();
       celist = celist->next;
@@ -1768,7 +1783,7 @@ yyreduce:
   case 26:
 
 /* Line 1806 of yacc.c  */
-#line 240 "keydbcfg-parser.y"
+#line 242 "keydbcfg-parser.y"
     {
       celist->next = new_title_entry_list();
       celist = celist->next;
@@ -1778,7 +1793,7 @@ yyreduce:
   case 29:
 
 /* Line 1806 of yacc.c  */
-#line 253 "keydbcfg-parser.y"
+#line 255 "keydbcfg-parser.y"
     {
       add_entry(celist, ENTRY_TYPE_DISCID, (yyvsp[(1) - (3)].string));
       add_entry(celist, ENTRY_TYPE_TITLE, (yyvsp[(3) - (3)].string));
@@ -1788,7 +1803,7 @@ yyreduce:
   case 42:
 
 /* Line 1806 of yacc.c  */
-#line 285 "keydbcfg-parser.y"
+#line 287 "keydbcfg-parser.y"
     {
       add_date_entry(celist, (yyvsp[(2) - (6)].digit), (yyvsp[(4) - (6)].digit), (yyvsp[(6) - (6)].digit));
     }
@@ -1797,7 +1812,7 @@ yyreduce:
   case 43:
 
 /* Line 1806 of yacc.c  */
-#line 292 "keydbcfg-parser.y"
+#line 294 "keydbcfg-parser.y"
     {
       add_entry(celist, ENTRY_TYPE_MEK, (yyvsp[(2) - (2)].string));
     }
@@ -1806,7 +1821,7 @@ yyreduce:
   case 44:
 
 /* Line 1806 of yacc.c  */
-#line 299 "keydbcfg-parser.y"
+#line 301 "keydbcfg-parser.y"
     {
       add_entry(celist, ENTRY_TYPE_VID, (yyvsp[(2) - (2)].string));
     }
@@ -1815,7 +1830,7 @@ yyreduce:
   case 45:
 
 /* Line 1806 of yacc.c  */
-#line 306 "keydbcfg-parser.y"
+#line 308 "keydbcfg-parser.y"
     {
       dkplist = NULL;
     }
@@ -1824,11 +1839,11 @@ yyreduce:
   case 48:
 
 /* Line 1806 of yacc.c  */
-#line 318 "keydbcfg-parser.y"
+#line 320 "keydbcfg-parser.y"
     {
       if (!dkplist)
       {
-        dkplist = (digit_key_pair_list *)malloc(sizeof(*dkplist));
+        dkplist = new_digit_key_pair_list();
         celist->entry.bn = dkplist;
       }
       dkplist = add_digit_key_pair_entry(dkplist, ENTRY_TYPE_BN, (yyvsp[(1) - (3)].digit), (yyvsp[(3) - (3)].string));
@@ -1838,7 +1853,7 @@ yyreduce:
   case 49:
 
 /* Line 1806 of yacc.c  */
-#line 330 "keydbcfg-parser.y"
+#line 332 "keydbcfg-parser.y"
     {
       add_entry(celist, ENTRY_TYPE_VUK, (yyvsp[(2) - (2)].string));
     }
@@ -1847,7 +1862,7 @@ yyreduce:
   case 50:
 
 /* Line 1806 of yacc.c  */
-#line 337 "keydbcfg-parser.y"
+#line 339 "keydbcfg-parser.y"
     {
       dkplist = NULL;
     }
@@ -1856,11 +1871,11 @@ yyreduce:
   case 53:
 
 /* Line 1806 of yacc.c  */
-#line 349 "keydbcfg-parser.y"
+#line 351 "keydbcfg-parser.y"
     {
       if (!dkplist)
       {
-        dkplist = (digit_key_pair_list *)malloc(sizeof(*dkplist));
+        dkplist = new_digit_key_pair_list();
         celist->entry.pak = dkplist;
       }
       dkplist = add_digit_key_pair_entry(dkplist, ENTRY_TYPE_PAK, (yyvsp[(1) - (3)].digit), (yyvsp[(3) - (3)].string));
@@ -1870,7 +1885,7 @@ yyreduce:
   case 54:
 
 /* Line 1806 of yacc.c  */
-#line 361 "keydbcfg-parser.y"
+#line 363 "keydbcfg-parser.y"
     {
       dkplist = NULL;
     }
@@ -1879,11 +1894,11 @@ yyreduce:
   case 57:
 
 /* Line 1806 of yacc.c  */
-#line 373 "keydbcfg-parser.y"
+#line 375 "keydbcfg-parser.y"
     {
       if (!dkplist)
       {
-        dkplist = (digit_key_pair_list *)malloc(sizeof(*dkplist));
+        dkplist = new_digit_key_pair_list();
         celist->entry.tk = dkplist;
       }
       dkplist = add_digit_key_pair_entry(dkplist, ENTRY_TYPE_TK, (yyvsp[(1) - (3)].digit), (yyvsp[(3) - (3)].string));
@@ -1893,7 +1908,7 @@ yyreduce:
   case 58:
 
 /* Line 1806 of yacc.c  */
-#line 385 "keydbcfg-parser.y"
+#line 387 "keydbcfg-parser.y"
     {
       dkplist = NULL;
     }
@@ -1902,11 +1917,11 @@ yyreduce:
   case 61:
 
 /* Line 1806 of yacc.c  */
-#line 397 "keydbcfg-parser.y"
+#line 399 "keydbcfg-parser.y"
     {
       if (!dkplist)
       {
-        dkplist = (digit_key_pair_list *)malloc(sizeof(*dkplist));
+        dkplist = new_digit_key_pair_list();
         celist->entry.uk = dkplist;
       }
       dkplist = add_digit_key_pair_entry(dkplist, ENTRY_TYPE_UK, (yyvsp[(1) - (3)].digit), (yyvsp[(3) - (3)].string));
@@ -1916,7 +1931,7 @@ yyreduce:
   case 62:
 
 /* Line 1806 of yacc.c  */
-#line 409 "keydbcfg-parser.y"
+#line 411 "keydbcfg-parser.y"
     {
       char *str = (char*)malloc(strlen((yyvsp[(1) - (2)].string)) + strlen((yyvsp[(2) - (2)].string)) + 1);
       strcpy(str, (yyvsp[(1) - (2)].string));
@@ -1929,7 +1944,7 @@ yyreduce:
   case 63:
 
 /* Line 1806 of yacc.c  */
-#line 417 "keydbcfg-parser.y"
+#line 419 "keydbcfg-parser.y"
     {
       char *str = (char*)malloc(strlen((yyvsp[(1) - (1)].string)) + 1);
       strcpy(str, (yyvsp[(1) - (1)].string));
@@ -1940,7 +1955,7 @@ yyreduce:
 
 
 /* Line 1806 of yacc.c  */
-#line 1944 "keydbcfg-parser.c"
+#line 1959 "keydbcfg-parser.c"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2171,7 +2186,7 @@ yyreturn:
 
 
 /* Line 2067 of yacc.c  */
-#line 423 "keydbcfg-parser.y"
+#line 425 "keydbcfg-parser.y"
 
 /* Function to parse a config file */
 int keydbcfg_parse_config(config_file *cfgfile, const char *path)
@@ -2201,6 +2216,8 @@ int keydbcfg_parse_config(config_file *cfgfile, const char *path)
   cfgfile->host_cert_list = head_clist;
   cfgfile->list = head_celist;
 
+  fclose(fp);
+
   if (retval)
     return 0;
 
@@ -2208,11 +2225,10 @@ int keydbcfg_parse_config(config_file *cfgfile, const char *path)
 }
 
 /* Function that returns pointer to new config file object */
-config_file *keydbcfg_new_config_file()
+config_file *keydbcfg_new_config_file(void)
 {
   config_file *cfgfile = (config_file *)malloc(sizeof(*cfgfile));
-  cfgfile->pkl = NULL;
-  cfgfile->list = NULL;
+  memset(cfgfile, 0, sizeof(*cfgfile));
   return cfgfile;
 }
 
@@ -2228,6 +2244,15 @@ int keydbcfg_config_file_close(config_file *cfgfile)
     cfgfile->pkl = next;
   }
 
+  /* free dk list */
+  while (cfgfile->dkl)
+  {
+    dk_list *next = cfgfile->dkl->next;
+    X_FREE(cfgfile->dkl->key);
+    X_FREE(cfgfile->dkl);
+    cfgfile->dkl = next;
+  }
+
   /* free host cert list */
   while (cfgfile->host_cert_list)
   {
@@ -2264,17 +2289,15 @@ int keydbcfg_config_file_close(config_file *cfgfile)
 }
 
 /* Function to return new dk_list object */
-static dk_list *new_dk_list()
+static dk_list *new_dk_list(void)
 {
   dk_list *dkl = (dk_list *)malloc(sizeof(*dkl));
-  dkl->key = NULL;
-  dkl->next = NULL;
+  memset(dkl, 0, sizeof(*dkl));
   return dkl;
 }
 
 /* Function to add dk to config file */
-static dk_list *add_dk_list_entry(dk_list *list, const char *key,
-                                  const char *node)
+static dk_list *add_dk_list_entry(dk_list *list, char *key, char *node)
 {
   if (!list)
   {
@@ -2282,26 +2305,25 @@ static dk_list *add_dk_list_entry(dk_list *list, const char *key,
     return NULL;
   }
 
-  list->key = (char*)malloc(strlen(key) + 1);
-  strcpy(list->key, key);
+  list->key  = key;
   list->node = strtoul(node, NULL, 16);
-
   list->next = new_dk_list();
 
+  X_FREE(node);
+
   return list->next;
 }
 
 /* Function to return new pk_list object */
-static pk_list *new_pk_list()
+static pk_list *new_pk_list(void)
 {
   pk_list *pkl = (pk_list *)malloc(sizeof(*pkl));
-  pkl->key = NULL;
-  pkl->next = NULL;
+  memset(pkl, 0, sizeof(*pkl));
   return pkl;
 }
 
 /* Function to add pk to config file */
-static pk_list *add_pk_list_entry(pk_list *list, const char *key)
+static pk_list *add_pk_list_entry(pk_list *list, char *key)
 {
   if (!list)
   {
@@ -2309,16 +2331,14 @@ static pk_list *add_pk_list_entry(pk_list *list, const char *key)
     return NULL;
   }
 
-  list->key = (char*)malloc(strlen(key) + 1);
-  strcpy(list->key, key);
-
+  list->key  = key;
   list->next = new_pk_list();
 
   return list->next;
 }
 
 /* Function to create new certificate list */
-static cert_list *new_cert_list()
+static cert_list *new_cert_list(void)
 {
   cert_list *list = (cert_list *)malloc(sizeof(*list));
   if (!list)
@@ -2327,19 +2347,15 @@ static cert_list *new_cert_list()
     return NULL;
   }
 
-  list->host_priv_key = NULL;
-  list->host_cert = NULL;
-  list->host_nonce = NULL;
-  list->host_key_point = NULL;
-  list->next = NULL;
+  memset(list, 0, sizeof(*list));
 
   return list;
 }
 
 /* Function to add certificate list entry into config file object */
-static cert_list *add_cert_list(cert_list *list, const char *host_priv_key,
-                         const char *host_cert, const char *host_nonce,
-                         const char *host_key_point)
+static cert_list *add_cert_list(cert_list *list, char *host_priv_key,
+                         char *host_cert, char *host_nonce,
+                         char *host_key_point)
 {
   if (!list)
   {
@@ -2347,14 +2363,10 @@ static cert_list *add_cert_list(cert_list *list, const char *host_priv_key,
     return NULL;
   }
 
-  list->host_priv_key = (char*)malloc(strlen(host_priv_key) + 1);
-  strcpy(list->host_priv_key, host_priv_key);
-  list->host_cert = (char*)malloc(strlen(host_cert) + 1);
-  strcpy(list->host_cert, host_cert);
-  list->host_nonce = (char*)malloc(strlen(host_nonce) + 1);
-  strcpy(list->host_nonce, host_nonce);
-  list->host_key_point = (char*)malloc(strlen(host_key_point) + 1);
-  strcpy(list->host_key_point, host_key_point);
+  list->host_priv_key = host_priv_key;
+  list->host_cert = host_cert;
+  list->host_nonce = host_nonce;
+  list->host_key_point = host_key_point;
 
   list->next = new_cert_list();
 
@@ -2362,7 +2374,7 @@ static cert_list *add_cert_list(cert_list *list, const char *host_priv_key,
 }
 
 /* Function that returns pointer to new title entry list */
-title_entry_list *new_title_entry_list()
+title_entry_list *new_title_entry_list(void)
 {
   title_entry_list *list = (title_entry_list *)malloc(sizeof(*list));
   if (!list)
@@ -2371,25 +2383,13 @@ title_entry_list *new_title_entry_list()
     return NULL;
   }
 
-  list->entry.discid = NULL;
-  list->entry.title = NULL;
-  list->entry.date.year = 0;
-  list->entry.date.month = 0;
-  list->entry.date.day = 0;
-  list->entry.mek = NULL;
-  list->entry.vid = NULL;
-  list->entry.bn = NULL;
-  list->entry.vuk = NULL;
-  list->entry.pak = NULL;
-  list->entry.tk = NULL;
-  list->entry.uk = NULL;
-  list->next = NULL;
+  memset(list, 0, sizeof(*list));
 
   return list;
 }
 
 /* Function to add standard string entries to a config entry */
-static int add_entry(title_entry_list *list, int type, const char *entry)
+static int add_entry(title_entry_list *list, int type, char *entry)
 {
   if (!list)
   {
@@ -2400,31 +2400,33 @@ static int add_entry(title_entry_list *list, int type, const char *entry)
   switch (type)
   {
     case ENTRY_TYPE_DISCID:
-      list->entry.discid = (char*)malloc(strlen(entry) + 1);
-      strcpy(list->entry.discid, entry);
+      X_FREE(list->entry.discid);
+      list->entry.discid = entry;
       break;
 
     case ENTRY_TYPE_TITLE:
+      X_FREE(list->entry.title);
       list->entry.title = (char*)malloc(strlen(entry) + 1);
       strcpy(list->entry.title, entry);
       break;
 
     case ENTRY_TYPE_MEK:
-      list->entry.mek = (char*)malloc(strlen(entry) + 1);
-      strcpy(list->entry.mek, entry);
+      X_FREE(list->entry.mek);
+      list->entry.mek = entry;
       break;
 
     case ENTRY_TYPE_VID:
-      list->entry.vid = (char*)malloc(strlen(entry) + 1);
-      strcpy(list->entry.vid, entry);
+      X_FREE(list->entry.vid);
+      list->entry.vid = entry;
       break;
 
     case ENTRY_TYPE_VUK:
-      list->entry.vuk = (char*)malloc(strlen(entry) + 1);
-      strcpy(list->entry.vuk, entry);
+      X_FREE(list->entry.vuk);
+      list->entry.vuk = entry;
       break;
 
     default:
+      X_FREE(entry);
       printf("WARNING: entry type passed in unknown\n");
       return 0;
   }
@@ -2433,7 +2435,7 @@ static int add_entry(title_entry_list *list, int type, const char *entry)
 }
 
 /* Function that returns pointer to new digit key pair list */
-static digit_key_pair_list *new_digit_key_pair_list()
+static digit_key_pair_list *new_digit_key_pair_list(void)
 {
   digit_key_pair_list *list = (digit_key_pair_list *)malloc(sizeof(*list));
   if (!list)
@@ -2442,16 +2444,14 @@ static digit_key_pair_list *new_digit_key_pair_list()
     return NULL;
   }
 
-  list->key_pair.digit = 0;
-  list->key_pair.key = NULL;
-  list->next = NULL;
+  memset(list, 0, sizeof(*list));
 
   return list;
 }
 
 /* Function used to add a digit/key pair to a list of digit key pair entries */
 static digit_key_pair_list *add_digit_key_pair_entry(digit_key_pair_list *list,
-                              int type, unsigned int digit, const char *key)
+                              int type, unsigned int digit, char *key)
 {
   if (!list)
   {
@@ -2460,8 +2460,7 @@ static digit_key_pair_list *add_digit_key_pair_entry(digit_key_pair_list *list,
   }
 
   list->key_pair.digit = digit;
-  list->key_pair.key = (char*)malloc(strlen(key) + 1);
-  strcpy(list->key_pair.key, key);
+  list->key_pair.key = key;
 
   list->next = new_digit_key_pair_list();
 
