@@ -52,6 +52,8 @@ struct aacs {
 
     /* VID is cached for BD-J */
     uint8_t   vid[16];
+    /* PMSN is cached for BD-J */
+    uint8_t   pmsn[16];
 
     /* unit key for each CPS unit */
     uint32_t  num_uks;
@@ -207,14 +209,14 @@ static MKB *_get_hrl_mkb(MMC *mmc)
 {
     MKB     *mkb = NULL;
     uint8_t *data;
-    int      size;
+    int      mkb_size;
 
-    data = mmc_read_mkb(mmc, 0, &size);
+    data = mmc_read_mkb(mmc, 0, &mkb_size);
 
     /* check acquired hrl signature */
-    if (data && size > 0) {
-        if (_rl_verify_signature(data, size)) {
-            mkb = mkb_init(data, size);
+    if (data && mkb_size > 0) {
+        if (_rl_verify_signature(data, mkb_size)) {
+            mkb = mkb_init(data, mkb_size);
             DEBUG(DBG_AACS, "Partial hrl mkb read. Version: %d\n", mkb_version(mkb));
         } else {
             DEBUG(DBG_AACS | DBG_CRIT, "invalid host revocation list signature, not using it\n");
@@ -299,7 +301,7 @@ static int _read_vid(AACS *aacs, cert_list *hcl)
         DEBUG(DBG_AACS, "Trying host certificate (id 0x%s)...\n",
               print_hex(tmp_str, cert + 4, 6));
 
-        int mmc_result = mmc_read_vid(mmc, priv_key, cert, aacs->vid);
+        int mmc_result = mmc_read_vid(mmc, priv_key, cert, aacs->vid, aacs->pmsn);
         switch (mmc_result) {
             case MMC_SUCCESS:
                 mkb_close(hrl_mkb);
@@ -323,7 +325,7 @@ static int _read_vid(AACS *aacs, cert_list *hcl)
 }
 
 static int _calc_vuk(AACS *aacs, uint8_t *mk, uint8_t *vuk,
-                     pk_list *pk_list, cert_list *host_cert_list)
+                     pk_list *pkl, cert_list *host_cert_list)
 {
     int error_code;
 
@@ -340,7 +342,7 @@ static int _calc_vuk(AACS *aacs, uint8_t *mk, uint8_t *vuk,
     }
 
     /* make sure we have media key */
-    error_code = _calc_mk(aacs, mk, pk_list);
+    error_code = _calc_mk(aacs, mk, pkl);
     if (error_code != AACS_SUCCESS) {
         return error_code;
     }
@@ -496,7 +498,6 @@ static void _find_config_entry(AACS *aacs, title_entry_list *ce,
                 hexstring_to_hex_array(aacs->uks + (16 * (aacs->num_uks - 1)), 16,
                                       ukcursor->key_pair.key);
 
-                char str[40];
                 DEBUG(DBG_AACS, "Unit key %d from keydb entry: %s\n",
                       aacs->num_uks,
                       print_hex(str, aacs->uks + (16 * (aacs->num_uks - 1)), 16));
@@ -749,9 +750,9 @@ AACS *aacs_open2(const char *path, const char *configfile_path, int *error_code)
     *error_code = _calc_uks(aacs, configfile_path);
 
     if (*error_code == AACS_SUCCESS) {
-        DEBUG(DBG_AACS, "AACS initialized! (%p)\n", aacs);
+        DEBUG(DBG_AACS, "AACS initialized!\n");
     } else {
-        DEBUG(DBG_AACS, "Failed to initialize AACS! (%p)\n", aacs);
+        DEBUG(DBG_AACS, "Failed to initialize AACS!\n");
     }
 
     return aacs;
@@ -766,7 +767,7 @@ void aacs_close(AACS *aacs)
     X_FREE(aacs->cps_units);
     X_FREE(aacs->path);
 
-    DEBUG(DBG_AACS, "AACS destroyed! (%p)\n", aacs);
+    DEBUG(DBG_AACS, "AACS destroyed!\n");
 
     X_FREE(aacs);
 }
@@ -792,7 +793,7 @@ int aacs_decrypt_unit(AACS *aacs, uint8_t *buf)
         return 1;
     }
 
-    DEBUG(DBG_AACS, "Failed decrypting unit [6144 bytes] (%p)\n", aacs);
+    DEBUG(DBG_AACS, "Failed decrypting unit [6144 bytes]\n");
 
     return 0;
 }
@@ -827,12 +828,25 @@ const uint8_t *aacs_get_vid(AACS *aacs)
     return aacs->vid;
 }
 
-static AACS_RL_ENTRY *_get_rl(const char *type, int *num_records, int *mkb_version)
+const uint8_t *aacs_get_pmsn(AACS *aacs)
+{
+    if (!memcmp(aacs->vid, empty_key, 16)) {
+        config_file *cf = keydbcfg_config_load(NULL);
+        if (cf) {
+            _read_vid(aacs, cf->host_cert_list);
+
+            keydbcfg_config_file_close(cf);
+        }
+    }
+    return aacs->pmsn;
+}
+
+static AACS_RL_ENTRY *_get_rl(const char *type, int *num_records, int *mkbv)
 {
     uint32_t len, version;
     void *data = NULL;
 
-    *num_records = *mkb_version = 0;
+    *num_records = *mkbv = 0;
 
     cache_get(type, &version, &len, NULL);
 
@@ -841,7 +855,7 @@ static AACS_RL_ENTRY *_get_rl(const char *type, int *num_records, int *mkb_versi
         if (cache_get(type, &version, &len, data) && len > 24) {
 
             if (_rl_verify_signature(data, len)) {
-                *mkb_version = version;
+                *mkbv = version;
                 *num_records = MKINT_BE32((uint8_t*)data + 20);
                 memmove(data, (uint8_t*)data + 24, len - 24);
 
@@ -863,14 +877,14 @@ static AACS_RL_ENTRY *_get_rl(const char *type, int *num_records, int *mkb_versi
     return data;
 }
 
-AACS_RL_ENTRY *aacs_get_hrl(int *num_records, int *mkb_version)
+AACS_RL_ENTRY *aacs_get_hrl(int *num_records, int *mkbv)
 {
-    return _get_rl("hrl", num_records, mkb_version);
+    return _get_rl("hrl", num_records, mkbv);
 }
 
-AACS_RL_ENTRY *aacs_get_drl(int *num_records, int *mkb_version)
+AACS_RL_ENTRY *aacs_get_drl(int *num_records, int *mkbv)
 {
-    return _get_rl("drl", num_records, mkb_version);
+    return _get_rl("drl", num_records, mkbv);
 }
 
 void aacs_select_title(AACS *aacs, uint32_t title)
@@ -880,22 +894,22 @@ void aacs_select_title(AACS *aacs, uint32_t title)
     }
 
     if (!aacs->cps_units) {
-        DEBUG(DBG_AACS|DBG_CRIT, "aacs_select_title(): CPS units not read ! (%p)\n", aacs);
+        DEBUG(DBG_AACS|DBG_CRIT, "aacs_select_title(): CPS units not read !\n");
         return;
     }
 
     if (title == 0xffff) {
         /* first play */
         aacs->current_cps_unit = aacs->cps_units[0];
-        DEBUG(DBG_AACS, "aacs_set_title(first_play): CPS unit %d (%p)\n", aacs->current_cps_unit, aacs);
+        DEBUG(DBG_AACS, "aacs_set_title(first_play): CPS unit %d\n", aacs->current_cps_unit);
         return;
     }
 
     if (title <= aacs->num_titles) {
         aacs->current_cps_unit = aacs->cps_units[title + 1];
-        DEBUG(DBG_AACS, "aacs_set_title(%d): CPS unit %d (%p)\n", title, aacs->current_cps_unit, aacs);
+        DEBUG(DBG_AACS, "aacs_set_title(%d): CPS unit %d\n", title, aacs->current_cps_unit);
         return;
     }
 
-    DEBUG(DBG_AACS|DBG_CRIT, "aacs_set_title(%d): invalid title ! (%p)\n", title, aacs);
+    DEBUG(DBG_AACS|DBG_CRIT, "aacs_set_title(%d): invalid title !\n", title);
 }
